# -*- ruby encoding: utf-8 -*-
#############################################################
# This file was automatically generated on 2012-09-26.      #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generator git on tinkerforge.com                   #
#############################################################

module Tinkerforge
  # Device for controlling up to 4 optically coupled digital outputs
  class BrickletIndustrialDigitalOut4 < Device

    # This callback is triggered whenever a monoflop timer reaches 0. The
    # parameters contain the involved pins and the current value of the pins
    # (the value after the monoflop).
    CALLBACK_MONOFLOP_DONE = 8

    FUNCTION_SET_VALUE = 1 # :nodoc:
    FUNCTION_GET_VALUE = 2 # :nodoc:
    FUNCTION_SET_MONOFLOP = 3 # :nodoc:
    FUNCTION_GET_MONOFLOP = 4 # :nodoc:
    FUNCTION_SET_GROUP = 5 # :nodoc:
    FUNCTION_GET_GROUP = 6 # :nodoc:
    FUNCTION_GET_AVAILABLE_FOR_GROUP = 7 # :nodoc:

    # Creates an object with the unique device ID <tt>uid</tt>. This object can
    # then be added to the IP connection.
    def initialize(uid)
      super uid

      @expected_name = 'Industrial Digital Out 4 Bricklet'

      @binding_version = [1, 0, 0]

      @callback_formats[CALLBACK_MONOFLOP_DONE] = 'S S'
    end

    # Sets the output value with a bitmask. The bitmask
    # is 16 bit long, *true* refers to high and *false* refers to 
    # low.
    # 
    # For example: The value 0b0000000000000011 will turn pins 0-1 
    # high and the other pins low.
    # 
    # If no groups are used (see BrickletIndustrialDigitalOut4#set_group), the pins correspond to the
    # markings on the Digital Out 4 Bricklet.
    # 
    # If groups are used, the pins correspond to the element in the group.
    # Element 1 in the group will get pins 0-3, element 2 pins 4-7, element 3
    # pins 8-11 and element 4 pins 12-15.
    def set_value(value_mask)
      send_request(FUNCTION_SET_VALUE, [value_mask], 'S', 0, '')
    end

    # Returns the bitmask as set by BrickletIndustrialDigitalOut4#set_value.
    def get_value
      send_request(FUNCTION_GET_VALUE, [], '', 2, 'S')
    end

    # Configures a monoflop of the pins specified by the first parameter
    # bitmask.
    # 
    # The second parameter is a bitmask with the desired value of the specified
    # pins (*true* means high and *false* means low).
    # 
    # The third parameter indicates the time (in ms) that the pins should hold
    # the value.
    # 
    # If this function is called with the parameters 
    # ((1 << 0) | (1 << 3), (1 << 0), 1500):
    # Pin 0 will get high and pin 3 will get low. In 1.5s pin 0 will get low and
    # pin 3 will get high again.
    # 
    # A monoflop can be used as a fail-safe mechanism. For example: Lets assume you
    # have a RS485 bus and a Digital Out 4 Bricklet connected to one of the slave
    # stacks. You can now call this function every second, with a time parameter
    # of two seconds and pin 0 high. Pin 0 will be high all the time. If now
    # the RS485 connection is lost, then pin 0 will turn low in at most two seconds.
    def set_monoflop(pin_mask, value_mask, time)
      send_request(FUNCTION_SET_MONOFLOP, [pin_mask, value_mask, time], 'S S L', 0, '')
    end

    # Returns (for the given pin) the current value and the time as set by
    # BrickletIndustrialDigitalOut4#set_monoflop as well as the remaining time until the value flips.
    # 
    # If the timer is not running currently, the remaining time will be returned
    # as 0.
    def get_monoflop(pin)
      send_request(FUNCTION_GET_MONOFLOP, [pin], 'C', 10, 'S L L')
    end

    # Sets a group of Digital Out 4 Bricklets that should work together. You can
    # find Bricklets that can be grouped together with BrickletIndustrialDigitalOut4#get_available_for_group.
    # 
    # The group consists of 4 elements. Element 1 in the group will get pins 0-3,
    # element 2 pins 4-7, element 3 pins 8-11 and element 4 pins 12-15.
    # 
    # Each element can either be one of the ports ('a' to 'd') or 'n' if it should
    # not be used.
    # 
    # For example: If you have two Digital Out 4 Bricklets connected to port A and
    # port B respectively, you could call with "['a', 'b', 'n', 'n']".
    # 
    # Now the pins on the Digital Out 4 on port A are assigned to 0-3 and the
    # pins on the Digital Out 4 on port B are assigned to 4-7. It is now possible
    # to call BrickletIndustrialDigitalOut4#set_value and control two Bricklets at the same time.
    def set_group(group)
      send_request(FUNCTION_SET_GROUP, [group], 'k4', 0, '')
    end

    # Returns the group as set by BrickletIndustrialDigitalOut4#set_group
    def get_group
      send_request(FUNCTION_GET_GROUP, [], '', 4, 'k4')
    end

    # Returns a bitmask of ports that are available for grouping. For example the
    # value 0b0101 means: Port *A* and Port *C* are connected to Bricklets that
    # can be grouped together.
    def get_available_for_group
      send_request(FUNCTION_GET_AVAILABLE_FOR_GROUP, [], '', 1, 'C')
    end

    # Registers a callback with ID <tt>cb</tt> to the block <tt>block</tt>.
    def register_callback(cb, &block)
      callback = block
      @registered_callbacks[cb] = callback
    end
  end
end
